<?php

namespace Form\FullForm;

use Form\FieldValidatorInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * In a html form, a FieldInterface represents an input, select or other form element identified by a
 * unique name attribute.
 */
interface FieldInterface
{
    const TYPE_TEXT_ID = 1;
    const TYPE_SELECT_ID = 2;
    const TYPE_RADIO_ID = 3;
    const TYPE_CHECKBOX_ID = 4;
    const TYPE_TEXTAREA_ID = 5;
    const TYPE_SUBMIT_ID = 6;
    const TYPE_FILE_ID = 7;

    /**
     * @return string
     */
    public function getName();

    /**
     * @return string
     */
    public function getLabel();

    /**
     * @return OptionInterface[]
     */
    public function getOptions();

    /**
     * @param string $value
     * @return OptionInterface|null
     */
    public function getOptionByValue($value);

    /**
     * @return OptionInterface|null
     */
    public function getSelectedValue();

    /**
     * @return int
     */
    public function getTypeId();

    /**
     * @return string|array
     */
    public function getValue();

    /**
     * Returns whether a field has been modified by the user
     * A dirty field CAN be displayed to the user as valid/invalid
     *
     * @return bool
     */
    public function isDirty();

    /**
     * Indicate that the field was incorrectly filled
     *
     * @return bool
     */
    public function hasError();

    /**
     * @return string
     */
    public function getError();

    /**
     * Indicate that the field was correctly filled
     *
     * @return bool
     */
    public function isValid();

    /**
     * The value of the type attribute for an input
     *
     * @return string|null
     */
    public function getInputType();

    /**
     * @return string|null
     */
    public function getPattern();

    /**
     * @return bool
     */
    public function isRequired();

    /**
     * @return bool
     */
    public function isAutocompleteOff();

    /**
     * Indicates whether there should be an autofocus attribute
     *
     * @return bool
     */
    public function isFocused();

    /**
     * @return bool
     */
    public function isDisabled();

    /**
     * @return bool
     */
    public function isReadOnly();

    /**
     * @return string
     */
    public function getPlaceHolder();

    /**
     * @return string[]
     */
    public function getTags();

    /**
     * @param string $tag
     * @return bool
     */
    public function hasTag($tag);
}
